/***************************************************************************\
*                                                                           *
* File: alloc.h                                                             *
*                                                                           *
* Desc: Header file for users of the dynamic memory management debugger.    *
*       This header has the interfaces to link the user to the debuging     *
*       routines for malloc, calloc, realloc, and free.  Also provided      *
*       are funcions:                                                       *
*                                                                           *
*          print_stats()  - Print current heap allcoation statistics.       *
*          memory_map()   - Print current allocation of memory.             *
*          check_heap()   - Walk the heap and check for errors.             *
*                                                                           *
* Vers: 3.00    Jim Roberts                                                 *
*                                                                           *
*       1.) Add function prototypes for open_log() and close_log().         *
*       2.) Interface all public functions to give __FILE__ and __LINE__.   *
*       3.) Encapsulate macro parameters to neutralize side effects.        *
*       4.) Add the allie_in_free() function.                               *
*                                                                           *
* Refs: Mike Daconta, "C Pointers and Dynamic Memory Management",  Chapter  *
*       13, Memory Management Internals, QED Publishing Group, Wellesley,   *
*       MA, 1993.                                                           *
*                                                                           *
\***************************************************************************/

#include "local.h"

/***************************************************************************\
*           P u b l i c   F u n c t i o n   P r o t o t y p e s             *
\***************************************************************************/

       void ptr_t * dbg_malloc      ( size_t size, const char * file,
                                         unsigned line );
       void ptr_t * dbg_calloc      ( size_t size, const char * file,
                                         unsigned line );
       void ptr_t * dbg_realloc     ( void ptr_t * ptr, size_t size,
                                         const char * file, unsigned line );
       void         dbg_free        ( void ptr_t * ptr, const char * file,
                                         unsigned line );
       char *       dbg_strdup      ( const char * str, const char * file,
                                         unsigned line );

       Boolean      dbg_aif         ( const char *, unsigned );
       void         dbg_prt_stats   ( const char *, unsigned );
       void         dbg_mem_map     ( const char *, unsigned );
       Boolean      dbg_chk_heap    ( const char *, unsigned );

       Boolean      open_log        ( void );
       void         log_comment     ( const char * );
       void         close_log       ( void );

/* These #define's remap the standard 'alloc.h' functions to the debuger.  */
/* For fun, 'strdup() from 'string.h' is maped to a debug version also.    */
/* The utility stuff is also replaced; this gives those calls a handle on  */
/* the file_name and line_number as well.                                  */

#define malloc(x)       dbg_malloc    ( (x), __FILE__, __LINE__ )
#define calloc(x)       dbg_calloc    ( (x), __FILE__, __LINE__ )
#define realloc(x,y)    dbg_realloc   ( (x), (y), __FILE__, __LINE__ )
#define free(x)         dbg_free      ( (x), __FILE__, __LINE__ )

#define strdup(x)       dbg_strdup    ( (x), __FILE__, __LINE__ )

#define allie_in_free() dbg_aif       ( __FILE__, __LINE__ )
#define print_stats()   dbg_prt_stats ( __FILE__, __LINE__ )
#define memory_map()    dbg_mem_map   ( __FILE__, __LINE__ )
#define check_heap()    dbg_chk_heap  ( __FILE__, __LINE__ )

/* End of File */